@@ -75,11 +75,26 @@ impl Kubectl {
         Ok(text)
     }
 
+    #[cfg(feature = "runtime")]
     pub async fn create_global_crd(
         &self,
         crd: apiextensionsv1::CustomResourceDefinition,
-    ) -> kube::Result<apiextensionsv1::CustomResourceDefinition> {
-        self.create_global_crd_impl(crd).await
+    ) -> anyhow::Result<apiextensionsv1::CustomResourceDefinition> {
+        use anyhow::Context;
+        use kube::runtime::wait;
+        use wait::conditions::is_crd_established;
+
+        let crd = self.create_global_crd_impl(crd).await?;
+        let name = crd
+            .metadata
+            .name
+            .as_ref()
+            .context("crd does not have name")?;
+
+        wait::await_condition(self.crds(), name, is_crd_established())
+            .await
+            .context("failed to established crd")?;
+        Ok(crd)
     }
 
     #[cfg(feature = "runtime")]
@@ -87,13 +102,16 @@ impl Kubectl {
         &self,
         crd: apiextensionsv1::CustomResourceDefinition,
     ) -> anyhow::Result<()> {
+        use anyhow::Context;
         use kube::runtime::wait;
         use wait::conditions::is_crd_established;
+
         let crd = self.ensure_global_k_is_installed(crd).await?;
         let name = crd.metadata.name.context("crd does not have name")?;
 
-        let establish = wait::await_condition(kubectl.crds(), &name, is_crd_established());
-        establish.await.context("failed to established crd")
+        wait::await_condition(self.crds(), &name, is_crd_established())
+            .await
+            .context("failed to established crd")
     }
 
     pub async fn ensure_global_k_is_installed<K>(&self, object: K) -> kube::Result<K>
