@@ -3,6 +3,7 @@
 // Use is subject to license terms.
 //
 
+use anyhow::Context;
 use serde_json as json;
 use tokio::time;
 
@@ -348,6 +349,26 @@ impl super::Kubectl {
         Err(kube::Error::Api(e))
     }
 
+    /// get all of the items `K` allow to filtered by `lp`
+    pub async fn list_all<K>(
+        client: kube::api::Api<K>,
+        mut lp: api::ListParams,
+    ) -> kube::Result<Vec<K>>
+    where
+        K: Clone + serde::de::DeserializeOwned + std::fmt::Debug,
+    {
+        let mut items = vec![];
+        loop {
+            let object_list = client.list(&lp).await?;
+            lp.continue_token = object_list.metadata.continue_;
+            items.extend(object_list.items);
+            if object_list.metadata.remaining_item_count.unwrap_or(0) == 0 {
+                break;
+            }
+        }
+        Ok(items)
+    }
+
     pub async fn update_config_map(
         &self,
         name: &str,
@@ -362,6 +383,69 @@ impl super::Kubectl {
         .await
     }
 
+    pub async fn allocate_in_configmap<K, V, F>(
+        &self,
+        name: &str,
+        key: K,
+        or_insert_with: F,
+    ) -> anyhow::Result<V>
+    where
+        K: Ord + ser::Serialize + de::DeserializeOwned + Clone,
+        V: ser::Serialize + de::DeserializeOwned + Clone,
+        F: Fn() -> anyhow::Result<V>,
+    {
+        use std::collections::btree_map::Entry::{Occupied, Vacant};
+        let mut value = None;
+        self.update_json_map(name, |cm_data: &mut BTreeMap<K, V>| {
+            let v = match cm_data.entry(key.clone()) {
+                Occupied(v) => Ok(v.get().clone()),
+                Vacant(e) => or_insert_with().map(|v| e.insert(v).clone()),
+            };
+            value = Some(v);
+        })
+        .await?;
+        value.context("value not allocated")?
+    }
+
+    pub async fn deallocate_in_configmap<K, V>(
+        &self,
+        name: &str,
+        key: &K,
+    ) -> anyhow::Result<Option<V>>
+    where
+        K: Ord + ser::Serialize + de::DeserializeOwned,
+        V: ser::Serialize + de::DeserializeOwned,
+    {
+        let mut value = None;
+
+        self.update_json_map(name, |cm_data: &mut BTreeMap<K, V>| {
+            value = cm_data.remove(key);
+        })
+        .await?;
+
+        Ok(value)
+    }
+
+    pub async fn update_json_map<K, V>(
+        &self,
+        name: &str,
+        mut update_fn: impl FnMut(&mut BTreeMap<K, V>),
+    ) -> anyhow::Result<corev1::ConfigMap>
+    where
+        K: Ord + ser::Serialize + de::DeserializeOwned,
+        V: ser::Serialize + de::DeserializeOwned,
+    {
+        // rv used to make sure that update has been applied successfully
+        let mut rv = Ok::<_, anyhow::Error>(());
+        let map = self
+            .update_config_map(name, |data| {
+                rv = try_update_json_configmap(data, &mut update_fn)
+            })
+            .await?;
+        rv?;
+        Ok(map)
+    }
+
     pub async fn list_daemonsets<'a>(
         &self,
         namespace: impl Into<Option<&'a str>>,
@@ -400,13 +484,14 @@ impl super::Kubectl {
         self.crds().list(&lp).await.map(|object| object.items)
     }
 
+    #[cfg(feature = "runtime")]
     pub(crate) async fn create_global_crd_impl(
         &self,
         crd: apiextensionsv1::CustomResourceDefinition,
     ) -> kube::Result<apiextensionsv1::CustomResourceDefinition> {
         use kube::Error::Api;
         let name = crd.name();
-        let crds = self.api::<apiextensionsv1::CustomResourceDefinition>();
+        let crds = self.crds();
         let conflict = http::StatusCode::CONFLICT.as_u16();
 
         let pp = self.post_params();
@@ -499,3 +584,37 @@ impl super::Kubectl {
         }
     }
 }
+
+/// an try-update function that takes a `confimap.data`: `BTreeMap<String, String>`
+/// and applies `update_fn: impl FnMut(&mut BTreeMap<K, V>)`
+fn try_update_json_configmap<K, V>(
+    cm_data: &mut BTreeMap<String, String>,
+    mut update_fn: impl FnMut(&mut BTreeMap<K, V>),
+) -> anyhow::Result<()>
+where
+    K: Ord + ser::Serialize + de::DeserializeOwned,
+    V: ser::Serialize + de::DeserializeOwned,
+{
+    use serde_json::{from_str, to_string};
+    use std::any::type_name;
+    let mut data: BTreeMap<K, V> = cm_data
+        .iter()
+        .map(|(key, value)| Ok((from_str(key)?, from_str(value)?)))
+        .collect::<serde_json::Result<_>>()
+        .with_context(|| {
+            format!(
+                "deserialize configmap {:?} data into BTreeMap<{}, {}>",
+                cm_data,
+                type_name::<K>(),
+                type_name::<V>(),
+            )
+        })?;
+    update_fn(&mut data);
+    *cm_data = data
+        .iter()
+        .map(|(key, value)| Ok((to_string(key)?, to_string(value)?)))
+        .collect::<serde_json::Result<_>>()
+        .with_context(|| format!("serialize configmap {:?} after update", cm_data))?;
+
+    Ok(())
+}
